@@ -25,6 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <dirent.h>
 #include <fcntl.h>
 #include <linux/types.h>
 #include <linux/mmc/ioctl.h>
@@ -64,9 +65,11 @@ struct rpmb_req {
 };
 #define RPMB_REQ_DATA(req) ((void *)((struct rpmb_req *)(req) + 1))
 
+#define RPMB_CID_SZ 16
+
 /* Response to device info request */
 struct rpmb_dev_info {
-	uint8_t cid[16];
+	uint8_t cid[RPMB_CID_SZ];
 	uint8_t rpmb_size_mult;	/* EXT CSD-slice 168: RPMB Size */
 	uint8_t rel_wr_sec_c;	/* EXT CSD-slice 222: Reliable Write Sector */
 				/*                    Count */
@@ -117,7 +120,6 @@ static pthread_mutex_t rpmb_mutex = PTHREAD_MUTEX_INITIALIZER;
 #define MMC_BLOCK_MAJOR	179
 
 /* mmc_ioc_cmd.opcode */
-#define MMC_SEND_EXT_CSD		 8
 #define MMC_READ_MULTIPLE_BLOCK		18
 #define MMC_WRITE_MULTIPLE_BLOCK	25
 
@@ -177,61 +179,214 @@ static int mmc_rpmb_fd(uint16_t dev_id)
 	return fd;
 }
 
-/* Open eMMC device dev_id */
-static int mmc_fd(uint16_t dev_id)
+/*
+ * Read @n bytes from @fd, takes care of short reads and EINTR.
+ * Adapted from “Advanced Programming In the UNIX Environment” by W. Richard
+ * Stevens and Stephen A. Rago, 2013, 3rd Edition, Addison-Wesley
+ * (EINTR handling was added)
+ */
+static ssize_t readn(int fd, void *ptr, size_t n)
 {
-	int fd = 0;
-	char path[PATH_MAX] = { 0 };
+	size_t nleft = n;
+	ssize_t nread = 0;
+	uint8_t *p = ptr;
 
-	DMSG("dev_id = %u", dev_id);
-#ifdef __ANDROID__
-	snprintf(path, sizeof(path), "/dev/block/mmcblk%u", dev_id);
-#else
-	snprintf(path, sizeof(path), "/dev/mmcblk%u", dev_id);
-#endif
-	fd = open(path, O_RDONLY);
-	if (fd < 0)
-		EMSG("Could not open %s (%s)", path, strerror(errno));
-
-	return fd;
+	while (nleft > 0) {
+		if ((nread = read(fd, p, nleft)) < 0) {
+			if (errno == EINTR)
+				continue;
+			if (nleft == n)
+				return -1; /* error, nothing read, return -1 */
+			else
+				break; /* error, return amount read so far */
+		} else if (nread == 0) {
+			break; /* EOF */
+		}
+		nleft -= nread;
+		p += nread;
+	}
+	return n - nleft; /* return >= 0 */
 }
 
-static void close_mmc_fd(int fd)
-{
-	close(fd);
-}
+/* Size of CID printed in hexadecimal */
+#define CID_STR_SZ (2 * RPMB_CID_SZ)
 
-/* Device Identification (CID) register is 16 bytes. It is read from sysfs. */
-static uint32_t read_cid(uint16_t dev_id, uint8_t *cid)
+static TEEC_Result read_cid_str(uint16_t dev_id, char cid[CID_STR_SZ + 1])
 {
 	TEEC_Result res = TEEC_ERROR_GENERIC;
 	char path[48] = { 0 };
-	char hex[3] = { 0 };
-	int st = 0;
 	int fd = 0;
-	int i = 0;
+	int st = 0;
 
 	snprintf(path, sizeof(path),
 		 "/sys/class/mmc_host/mmc%u/mmc%u:0001/cid", dev_id, dev_id);
 	fd = open(path, O_RDONLY);
+	if (fd < 0)
+		return TEEC_ERROR_ITEM_NOT_FOUND;
+	st = readn(fd, cid, CID_STR_SZ);
+	if (st != CID_STR_SZ) {
+		EMSG("Read CID error");
+		if (errno)
+			EMSG("%s", strerror(errno));
+		res = TEEC_ERROR_NO_DATA;
+		goto out;
+	}
+	res = TEEC_SUCCESS;
+out:
+	close(fd);
+	return res;
+}
+
+static int hexchar2int(char c)
+{
+	if (c >= '0' && c <= '9')
+		return c - '0';
+	if (c >= 'a' && c <= 'f')
+		return c - 'a' + 10;
+	if (c >= 'A' && c <= 'F')
+		return c - 'A' + 10;
+	return -1;
+}
+
+static int hexbyte2int(char *hex)
+{
+	int v1 = hexchar2int(hex[0]);
+	int v2 = hexchar2int(hex[1]);
+
+	if (v1 < 0 || v2 < 0)
+		return -1;
+	return 16 * v1 + v2;
+}
+
+/* Device Identification (CID) register is 16 bytes. It is read from sysfs. */
+static TEEC_Result read_cid(uint16_t dev_id, uint8_t *cid)
+{
+	TEEC_Result res = TEEC_ERROR_GENERIC;
+	char cid_str[CID_STR_SZ + 1] = { };
+	int i = 0;
+	int v = 0;
+
+	res = read_cid_str(dev_id, cid_str);
+	if (res)
+		return res;
+
+	for (i = 0; i < RPMB_CID_SZ; i++) {
+		v = hexbyte2int(cid_str + 2 * i);
+		if (v < 0) {
+			EMSG("Invalid CID string: %s", cid_str);
+			return TEEC_ERROR_NO_DATA;
+		}
+		cid[i] = v;
+	}
+	return TEEC_SUCCESS;
+}
+
+static TEEC_Result read_mmc_sysfs_hex(uint16_t dev_id, const char *file, uint8_t *value)
+{
+	TEEC_Result status = TEEC_SUCCESS;
+	char path[255] = { 0 };
+	char buf[255] = { 0 };
+	char *endp = buf;
+	int fd = 0;
+	int ret = 0;
+
+	snprintf(path, sizeof(path), "/sys/class/mmc_host/mmc%u/mmc%u:0001/%s",
+		 dev_id, dev_id, file);
+
+	fd = open(path, O_RDONLY);
 	if (fd < 0) {
 		EMSG("Could not open %s (%s)", path, strerror(errno));
 		return TEEC_ERROR_ITEM_NOT_FOUND;
 	}
 
-	for (i = 0; i < 16; i++) {
-		st = read(fd, hex, 2);
-		if (st < 0) {
-			EMSG("Read CID error (%s)", strerror(errno));
-			res = TEEC_ERROR_NO_DATA;
-			goto err;
-		}
-		cid[i] = (uint8_t)strtol(hex, NULL, 16);
+	ret = readn(fd, buf, sizeof(buf));
+	if (ret < 0) {
+		EMSG("Read error (%s)", strerror(errno));
+		status = TEEC_ERROR_NO_DATA;
+		goto out;
 	}
-	res = TEEC_SUCCESS;
-err:
+
+	errno = 0;
+	*value = strtol(buf, &endp, 16);
+	if (errno || endp == buf)
+		status = TEEC_ERROR_GENERIC;
+
+out:
 	close(fd);
-	return res;
+	return status;
+}
+
+static TEEC_Result read_size_mult(uint16_t dev_id, uint8_t *value)
+{
+	return read_mmc_sysfs_hex(dev_id, "raw_rpmb_size_mult", value);
+}
+
+static TEEC_Result read_rel_wr_sec_c(uint16_t dev_id, uint8_t *value)
+{
+	return read_mmc_sysfs_hex(dev_id, "rel_sectors", value);
+}
+
+/*
+ * - If --rpmb-cid is given, find the eMMC RPMB device number with the specified
+ * CID, cache the number, copy it to @ndev_id and return true. If not found
+ * return false.
+ * - If --rpmb-cid is not given, copy @dev_id to @ndev_id and return true.
+ */
+static bool remap_rpmb_dev_id(uint16_t dev_id, uint16_t *ndev_id)
+{
+	TEEC_Result res = TEEC_ERROR_GENERIC;
+	static bool found = false;
+	static bool err = false;
+	static uint16_t id = 0;
+	char cid[CID_STR_SZ + 1] = { };
+	struct dirent *dent = NULL;
+	DIR *dir = NULL;
+	int num = 0;
+
+	if (err || found)
+		goto out;
+
+	if (!supplicant_params.rpmb_cid) {
+		id = dev_id;
+		found = true;
+		goto out;
+	}
+
+	dir = opendir("/sys/class/mmc_host");
+	if (!dir) {
+		EMSG("Could not open /sys/class/mmc_host (%s)",
+		     strerror(errno));
+		err = true;
+		goto out;
+	}
+
+	while ((dent = readdir(dir))) {
+		if (sscanf(dent->d_name, "%*[^0123456789]%d", &num) != 1)
+			continue;
+		if (num > UINT16_MAX) {
+			EMSG("Too many MMC devices");
+			err = true;
+			break;
+		}
+		id = (uint16_t)num;
+		res = read_cid_str(id, cid);
+		if (res)
+			continue;
+		if (strcmp(cid, supplicant_params.rpmb_cid))
+			continue;
+		IMSG("RPMB device %s is at /dev/mmcblk%urpmb\n", cid, id);
+		found = true;
+		break;
+	}
+
+	closedir(dir);
+
+	if (!found)
+		err = true;
+out:
+	if (found)
+		*ndev_id = id;
+	return found;
 }
 
 #else /* RPMB_EMU */
@@ -485,12 +640,6 @@ static uint32_t read_cid(uint16_t dev_id, uint8_t *cid)
 	return TEEC_SUCCESS;
 }
 
-static void ioctl_emu_set_ext_csd(uint8_t *ext_csd)
-{
-	ext_csd[168] = EMU_RPMB_SIZE_MULT;
-	ext_csd[222] = EMU_RPMB_REL_WR_SEC_C;
-}
-
 /* A crude emulation of the MMC ioc commands we need for RPMB */
 static int ioctl_emu_cmd(int fd, struct mmc_ioc_cmd *cmd)
 {
@@ -502,10 +651,6 @@ static int ioctl_emu_cmd(int fd, struct mmc_ioc_cmd *cmd)
 		return -1;
 
 	switch (cmd->opcode) {
-	case MMC_SEND_EXT_CSD:
-		ioctl_emu_set_ext_csd((uint8_t *)(uintptr_t)cmd->data_ptr);
-		break;
-
 	case MMC_WRITE_MULTIPLE_BLOCK:
 		frm = (struct rpmb_data_frame *)(uintptr_t)cmd->data_ptr;
 		msg_type = ntohs(frm->msg_type);
@@ -610,43 +755,30 @@ static int mmc_rpmb_fd(uint16_t dev_id)
 	return 0;
 }
 
-static int mmc_fd(uint16_t dev_id)
+static TEEC_Result read_size_mult(uint16_t dev_id, uint8_t *value)
 {
 	(void)dev_id;
 
-	return 0;
+	*value = EMU_RPMB_SIZE_MULT;
+	return TEEC_SUCCESS;
 }
 
-static void close_mmc_fd(int fd)
+static TEEC_Result read_rel_wr_sec_c(uint16_t dev_id, uint8_t *value)
 {
-	(void)fd;
+	(void)dev_id;
+
+	*value = EMU_RPMB_REL_WR_SEC_C;
+	return TEEC_SUCCESS;
+}
+
+static bool remap_rpmb_dev_id(uint16_t dev_id, uint16_t *ndev_id)
+{
+	*ndev_id = dev_id;
+	return true;
 }
 
 #endif /* RPMB_EMU */
 
-/*
- * Extended CSD Register is 512 bytes and defines device properties
- * and selected modes.
- */
-static uint32_t read_ext_csd(int fd, uint8_t *ext_csd)
-{
-	int st = 0;
-	struct mmc_ioc_cmd cmd = {
-		.blksz = 512,
-		.blocks = 1,
-		.flags = MMC_RSP_R1 | MMC_CMD_ADTC,
-		.opcode = MMC_SEND_EXT_CSD,
-	};
-
-	mmc_ioc_cmd_set_data(cmd, ext_csd);
-
-	st = IOCTL(fd, MMC_IOC_CMD, &cmd);
-	if (st < 0)
-		return TEEC_ERROR_GENERIC;
-
-	return TEEC_SUCCESS;
-}
-
 static inline void set_mmc_io_cmd(struct mmc_ioc_cmd *cmd, unsigned int blocks,
 				  __u32 opcode, int write_flag)
 {
@@ -770,28 +902,26 @@ out:
 
 static uint32_t rpmb_get_dev_info(uint16_t dev_id, struct rpmb_dev_info *info)
 {
-	int fd = 0;
-	uint32_t res = 0;
-	uint8_t ext_csd[512] = { 0 };
+	TEEC_Result res = TEEC_SUCCESS;
+	uint8_t rpmb_size_mult = 0;
+	uint8_t rel_wr_sec_c = 0;
 
 	res = read_cid(dev_id, info->cid);
 	if (res != TEEC_SUCCESS)
 		return res;
 
-	fd = mmc_fd(dev_id);
-	if (fd < 0)
-		return TEEC_ERROR_BAD_PARAMETERS;
-
-	res = read_ext_csd(fd, ext_csd);
+	res = read_size_mult(dev_id, &rpmb_size_mult);
 	if (res != TEEC_SUCCESS)
-		goto err;
+		return res;
+	info->rpmb_size_mult = rpmb_size_mult;
+
+	res = read_rel_wr_sec_c(dev_id, &rel_wr_sec_c);
+	if (res != TEEC_SUCCESS)
+		return res;
+	info->rel_wr_sec_c = rel_wr_sec_c;
 
-	info->rel_wr_sec_c = ext_csd[222];
-	info->rpmb_size_mult = ext_csd[168];
 	info->ret_code = RPMB_CMD_GET_DEV_INFO_RET_OK;
 
-err:
-	close_mmc_fd(fd);
 	return res;
 }
 
@@ -806,17 +936,21 @@ static uint32_t rpmb_process_request_unlocked(void *req, size_t req_size,
 	struct rpmb_req *sreq = req;
 	size_t req_nfrm = 0;
 	size_t rsp_nfrm = 0;
+	uint16_t dev_id = 0;
 	uint32_t res = 0;
 	int fd = 0;
 
 	if (req_size < sizeof(*sreq))
 		return TEEC_ERROR_BAD_PARAMETERS;
 
+	if (!remap_rpmb_dev_id(sreq->dev_id, &dev_id))
+		return TEEC_ERROR_ITEM_NOT_FOUND;
+
 	switch (sreq->cmd) {
 	case RPMB_CMD_DATA_REQ:
 		req_nfrm = (req_size - sizeof(struct rpmb_req)) / 512;
 		rsp_nfrm = rsp_size / 512;
-		fd = mmc_rpmb_fd(sreq->dev_id);
+		fd = mmc_rpmb_fd(dev_id);
 		if (fd < 0)
 			return TEEC_ERROR_BAD_PARAMETERS;
 		res = rpmb_data_req(fd, RPMB_REQ_DATA(req), req_nfrm, rsp,
@@ -829,8 +963,7 @@ static uint32_t rpmb_process_request_unlocked(void *req, size_t req_size,
 			EMSG("Invalid req/rsp size");
 			return TEEC_ERROR_BAD_PARAMETERS;
 		}
-		res = rpmb_get_dev_info(sreq->dev_id,
-					(struct rpmb_dev_info *)rsp);
+		res = rpmb_get_dev_info(dev_id, (struct rpmb_dev_info *)rsp);
 		break;
 
 	default:
